module msamurai
   use precision
   use constants
   use options
   use ncuts
   use notfirst
   use ltest
   use madds
   use mgetqs
   use mgetbase
   use mrestore
   use mgetc5
   use mgetc4
   use mgetc3
   use mgetc2
   use mgetc1
   use mtests
   implicit none

contains

      subroutine initsamurai(ameth, asca, averb, atest)
      implicit none
      integer :: asca, averb, atest
      character*4 ameth

      call banner

      if ((ameth.eq.('diag')).or.(ameth.eq.('tree'))) then
      imeth = ameth
      else
      write(6,*) 'incompatible value for imeth'
      write(6,*) 'imeth =',ameth
      stop
      endif

      if ((asca.eq.1).or.(asca.eq.2)) then
      isca = asca
      else
      write(6,*) 'incompatible value for isca'
      write(6,*) 'isca =',asca
      stop
      endif

      if ((averb.ge.0).or.(averb.le.3)) then
      verbosity = averb
      else
      write(6,*) 'incompatible value for verbosity'
      write(6,*) 'verbosity =',averb
      stop
      endif

      if ((atest.ge.0).or.(atest.le.3)) then
      itest = atest
      else
      write(6,*) 'incompatible value for itest'
      write(6,*) 'itest =',atest
      stop
      endif

      if ((imeth.eq.'tree').and.(itest.gt.0)) itest=2
      if (isca .eq. 2) then
       call avh_olo_onshell(1.d-8)            
      else
       call qlinit
      end if

 901  format(a40,a4,a10,I1,a15,I1,a11,I1)

      if (verbosity.gt.0) then
      if (notfirstp.eqv.(.false.)) then
      open(unit=20,file='output.dat',status='unknown')
      write(20,*) '------------------------------------------------------------------&
     & ---------------'
      write(20,901) ' SAMURAI called with arguments: imeth = ',imeth,' ; isca = ',isca,&
     & ' ; verbosity = ',verbosity,' ; itest = ',itest
      write(20,*) '------------------------------------------------------------------&
     & ---------------'
      notfirstp = .TRUE.
      endif
      endif
      if (itest.gt.0) then
      if (notfirstd.eqv.(.false.)) then
      open(unit=30,file='bad.points',status='unknown')
      write(30,*) '--------------------------------------------------------------------'
      write(30,904) ' Points that have been discarded by SAMURAI because failing itest = ',itest
      write(30,*) '--------------------------------------------------------------------'
      notfirstd = .true.
      endif
      endif

 904  format(a68,I1)

      call rtlimit

  end subroutine initsamurai

      subroutine rtlimit
      implicit none
      real tpwlimit,tnnlimit,tnnlimit4,tnnlimit3,tnnlimit2,tnnlimit1
      open(unit=10,file='ltest.dat',status='old')

      read(10,*)
      read(10,*)
      read(10,*)  tpwlimit
      pwlimit   = real(tpwlimit, ki)
      read(10,*)  tnnlimit
      nnlimit   = real(tnnlimit, ki)
      read(10,*)  tnnlimit4
      lnnlimit4 = real(tnnlimit4, ki)
      read(10,*)  tnnlimit3
      lnnlimit3 = real(tnnlimit3, ki)
      read(10,*)  tnnlimit2
      lnnlimit2 = real(tnnlimit2, ki)
      read(10,*)  tnnlimit1
      lnnlimit1 = real(tnnlimit1, ki)

      close(10)

  end subroutine rtlimit

      subroutine exitsamurai
      implicit none

      if(verbosity.gt.0)then
       close(20)
      endif

      if(itest.gt.0)then
       close(30)
      endif

  end subroutine exitsamurai


   subroutine banner
      implicit none

      print*
      print*, ' ********************************************************************'
      print*, ' ********************** SAMURAI - version 1.0 ***********************'
      print*, ' ********************************************************************'
      print*, ' *                                                                  *'
      print*, ' * SAMURAI, v1.0                                     May 29th, 2010 *'
      print*, ' *                                                                  *'
      print*, ' * Authors: P. Mastrolia, G. Ossola, T. Reiter and F. Tramontano    *'
      print*, ' *                                                                  *'
      print*, ' * pierpaolo.mastrolia@cern.ch                                      *'
      print*, ' * gossola@citytech.cuny.edu                                        *'
      print*, ' * thomasr@nikhef.nl                                                *'
      print*, ' * francesco.tramontano@cern.ch                                     *'
      print*, ' *                                                                  *'
      print*, ' *  For details please see:                                         *'
      print*, ' *                                                                  *'
      print*, ' *  On the web:  http://cern.ch/samurai                             *'
      print*, ' *                                                                  *'
      print*, ' ********************************************************************'
      print*, ' *                                                                  *'
      print*, ' * output files: <output.log>   [ for verbosity.gt.0 ]              *'
      print*, ' *                                                                  *'
      print*, ' *               <bad.points>   [ for itest.gt.0     ]              *'
      print*, ' *                                                                  *'
      print*, ' ********************************************************************'

  end subroutine banner



   pure subroutine InitDenominators(nleg,Vi,msq,&
      &Q0,m0,Q1,m1,Q2,m2,Q3,m3,Q4,m4,Q5,m5,Q6,m6,Q7,m7)
      implicit none
      integer, intent(in) :: nleg
      real(ki), dimension(0:nleg-1,4), intent(out) :: Vi
      real(ki), dimension(0:nleg-1), intent(out) :: msq

      real(ki), dimension(4), intent(in), optional :: Q0
      real(ki), intent(in), optional :: m0
      real(ki), dimension(4), intent(in), optional :: Q1
      real(ki), intent(in), optional :: m1
      real(ki), dimension(4), intent(in), optional :: Q2
      real(ki), intent(in), optional :: m2
      real(ki), dimension(4), intent(in), optional :: Q3
      real(ki), intent(in), optional :: m3
      real(ki), dimension(4), intent(in), optional :: Q4
      real(ki), intent(in), optional :: m4
      real(ki), dimension(4), intent(in), optional :: Q5
      real(ki), intent(in), optional :: m5
      real(ki), dimension(4), intent(in), optional :: Q6
      real(ki), intent(in), optional :: m6
      real(ki), dimension(4), intent(in), optional :: Q7
      real(ki), intent(in), optional :: m7

      if(present(Q0) .and. present(m0)) then
         Vi(0,:) = Q0
         msq(0) = m0*m0
      end if
      if(present(Q1) .and. present(m1)) then
         Vi(1,:) = Q1
         msq(1) = m1*m1
      end if
      if(present(Q2) .and. present(m2)) then
         Vi(2,:) = Q2
         msq(2) = m2*m2
      end if
      if(present(Q3) .and. present(m3)) then
         Vi(3,:) = Q3
         msq(3) = m3*m3
      end if
      if(present(Q4) .and. present(m4)) then
         Vi(4,:) = Q4
         msq(4) = m4*m4
      end if
      if(present(Q5) .and. present(m5)) then
         Vi(5,:) = Q5
         msq(5) = m5*m5
      end if
      if(present(Q6) .and. present(m6)) then
         Vi(6,:) = Q6
         msq(6) = m6*m6
      end if
      if(present(Q7) .and. present(m7)) then
         Vi(7,:) = Q7
         msq(7) = m7*m7
      end if

   end  subroutine InitDenominators


      subroutine samurai(numeval,tot,totr,Vi,msq,nleg,rank,istop,scale2,ok)
      implicit none

      integer nleg
      integer i,j,k,j1,j2,j3,j4,j5,icut5,icut4,icut3,icut2,icut1
      integer cut5,cut4,cut3,cut2,cut1,n1,ep
      integer rank,diff,istop

      real(ki), dimension(0:nleg-1):: msq
      real(ki), dimension(0:nleg-1,4) ::Vi
      real(ki) :: scale2, mu2, mu2test, r1, r2
      real(ki), dimension(4):: k1, k2, k3, e1, e2, p0, L3

      complex(ki) :: c5,tot2r,tot3r,tot4r,totr
      complex(ki), dimension(4):: q5,  q41, q42, q43, q44, q45
      complex(ki), dimension(4):: q31, q32, q33, q34, q35
      complex(ki), dimension(4):: q36, q37, q38, q39, q310
      complex(ki), dimension(4):: q21, q22, q23, q24, q25
      complex(ki), dimension(4):: q26, q27, q28, q29, q210
      complex(ki), dimension(4):: q11, q12, q13, q14, q15
      complex(ki), dimension(4):: e3(4),e4(4), qt, qtest
      complex(ki), dimension(0:4) :: c4, c1
      complex(ki), dimension(0:9) :: c3, c2
      complex(ki), dimension(-2:0) :: tot, tot4, tot3, tot2, tot1
      logical :: ok

      interface
         function     numeval(ncut, Q, mu2)
           use precision
           implicit none
           integer, intent(in) :: ncut
           complex(ki), dimension(4), intent(in) :: Q
           real(ki), intent(in) :: mu2
           complex(ki) :: numeval
         end function numeval
      end interface

      if (nleg.gt.maxleg) then
      write(6,*) 'reduction called for nleg.gt.maxleg'
      write(6,*) 'maxleg, nleg ',maxleg,nleg
      write(6,*) 'please change the values max1,..,max5,maxleg in constants.f90'
      write(6,*) 'and compile again the library'
      stop
      endif

      diff = nleg-rank
      
      if(verbosity.gt.0)then
            write(20,*)
            write(20,*)
         write(20,*) 'Denominators: '
         do k=0,nleg-1
            write(20,902) ' Pi(',k,') = ',Vi(k,:)
            write(20,903) 'msq(',k,') = ',msq(k)
            write(20,*)
         enddo
         write(20,*)' '
      endif

      totr=czip
      tot4r=czip
      tot3r=czip
      tot2r=czip
     
      do i=-2,0
      tot(i)=czip
      tot4(i)=czip
      tot3(i)=czip
      tot2(i)=czip
      tot1(i)=czip
      enddo

      if      (nleg.ge.5) then
         goto 10
      elseif  (nleg.eq.4) then
         goto 20
      elseif  (nleg.eq.3) then
         goto 30
      elseif  (nleg.eq.2) then
         goto 40
      elseif  ((nleg.eq.1).or.(nleg.le.0)) then
      write(6,*) 'reduction called for tadpole or tree level diagram'
      write(6,*) 'nleg=',nleg
      stop
      endif

 10   continue

      if(verbosity.gt.0)then
         write(20,*) 'Pentagon coefficients: '
      endif

!--- Quintuple cuts
      icut5=1
      do j5=4,nleg-1
       do j4=3,j5-1
        do j3=2,j4-1
         do j2=1,j3-1
          do j1=0,j2-1

      cut5=j5*10000+j4*1000+j3*100+j2*10+j1

      do n1=1,4
         k1(n1)=Vi(j2,n1)-Vi(j1,n1)
         k2(n1)=Vi(j1,n1)-Vi(j5,n1)
         p0(n1)=Vi(j1,n1)
      enddo

      call getbase(k1,k2,r1,r2,e1,e2,e3,e4)
      call getq5(nleg,cut5,e1,e2,e3,e4,p0,Vi,msq,r1,r2,q5,mu2)
      call getc5(numeval,nleg,c5,cut5,Vi,msq,q5,mu2)
      call store5(icut5,cut5,p0,e1,e2,e3,e4,c5)

      if(verbosity.gt.0)then
         write(20,9005) 'c5(', cut5,')  = (',real(c5),',',aimag(c5),'  )'
         write(20,*)
      endif

!--- nullify all
      r1=zip
      r2=zip
       do j=1,4
          e1(j)=0d0
          e2(j)=0d0
          e3(j)=czip
          e4(j)=czip
          k1(j)=0d0
          k2(j)=0d0
          p0(j)=0d0
       enddo

       icut5=icut5+1
          enddo
         enddo
        enddo
       enddo
      enddo
      nc5=icut5-1

      if (istop.eq.5) goto 99

 20   continue

      if(verbosity.gt.0)then
         write(20,*) 'Box coefficients: '
      endif
      

!--- Quadruple cuts
      icut4=1
      do j4=3,nleg-1
       do j3=2,j4-1
        do j2=1,j3-1
         do j1=0,j2-1

      cut4=j4*1000+j3*100+j2*10+j1

      do n1=1,4
         k1(n1)=Vi(j2,n1)-Vi(j1,n1)
         k2(n1)=Vi(j1,n1)-Vi(j4,n1)
         k3(n1)=Vi(j3,n1)-Vi(j1,n1)
         p0(n1)=Vi(j1,n1)
         L3(n1)=Vi(j4,n1)-Vi(j3,n1)
      enddo

      call getbase(k1,k2,r1,r2,e1,e2,e3,e4)
      call getq4(nleg,cut4,e1,e2,e3,e4,p0,k1,k2,k3,L3,r1,r2,&
     & q41,q42,q43,q44,q45,qt,msq)
      call getc4(numeval,nleg,rank,c4,cut4,q41,q42,q43,q44,q45,qt,&
     & p0,Vi,msq)
      if (itest.eq.2) call lnntest4(numeval,cut4,c4,qt,p0,L3,e3,e4,ok)
      call store4(icut4,cut4,L3,p0,e1,e2,e3,e4,c4)
      call add4(nleg,c4,cut4,Vi,msq,tot4,tot4r,scale2)

      if(verbosity.gt.0)then
         do i=0,4
         write(20,9004) 'c4(', cut4,',',i,') = (',real(c4(i)),',',aimag(c4(i)),'  )'
         enddo
      write(20,*)
      endif
      
      do ep=-2,0
      tot(ep)=tot(ep)+tot4(ep)
      enddo
      totr=totr+tot4r

!--- nullify all
      r1=zip
      r2=zip
       do j=1,4
          e1(j)=0d0
          e2(j)=0d0
          e3(j)=czip
          e4(j)=czip
          k1(j)=0d0
          k2(j)=0d0
          k3(j)=0d0
          p0(j)=0d0
          L3(j)=0d0
       enddo

       icut4=icut4+1
         enddo
        enddo
       enddo
      enddo
      nc4=icut4-1


      if (istop.eq.4) goto 99

 30   continue

      if (diff.ge.4.and.itest.ne.1) goto 93
      
      if(verbosity.gt.0.and.diff.le.3)then
         write(20,*) 'Triangle coefficients: '
      endif
      
!--- Triple cuts
      icut3=1
      do j3=2,nleg-1
       do j2=1,j3-1
        do j1=0,j2-1

      cut3=j3*100+j2*10+j1

      do n1=1,4
         k1(n1)=Vi(j2,n1)-Vi(j1,n1)
         k2(n1)=Vi(j1,n1)-Vi(j3,n1)
         p0(n1)=Vi(j1,n1)
      enddo

      call getbase(k1,k2,r1,r2,e1,e2,e3,e4)
      call getq3(nleg,cut3,e1,e2,e3,e4,p0,k1,k2,msq,r1,r2,&
     & q31,q32,q33,q34,q35,q36,q37,q38,q39,q310,qt)
      call getc3(numeval,nleg,rank,c3,cut3,q31,q32,q33,q34,q35,q36,q37,&
     & q38,q39,q310,qt,Vi,msq)
       if (itest.eq.2) call lnntest3(numeval,cut3,c3,qt,p0,e3,e4,ok)
      call store3(icut3,cut3,p0,e1,e2,e3,e4,c3)
      call add3(nleg,c3,cut3,Vi,msq,tot3,tot3r,scale2)

      if(verbosity.gt.0.and.diff.le.3)then
         do i=0,9
         write(20,9003) 'c3(',cut3,',',i,')  = (',real(c3(i)),',',aimag(c3(i)),'  )'
         enddo
      write(20,*)
      endif


      do ep=-2,0
      tot(ep)=tot(ep)+tot3(ep)
      enddo
      totr=totr+tot3r

!--- nullify all
      r1=zip
      r2=zip
       do j=1,4
          e1(j)=0d0
          e2(j)=0d0
          e3(j)=czip
          e4(j)=czip
          k1(j)=0d0
          k2(j)=0d0
          p0(j)=0d0
       enddo

       icut3=icut3+1
         enddo
        enddo
       enddo
      nc3=icut3-1

      if (istop.eq.3) goto 99

 40   continue

      if (diff.ge.3) goto 95
      
      if(verbosity.gt.0)then
         write(20,*) 'Bubble coefficients: '
      endif

!--- Double cuts
      icut2=1
      do j2=1,nleg-1
        do j1=0,j2-1

      cut2=j2*10+j1

      do n1=1,4
         k1(n1)=Vi(j2,n1)-Vi(j1,n1)
         p0(n1)=Vi(j1,n1)
      enddo

      k2(1)=-sign(one/rt3,k1(1))
      k2(2)=-sign(one/rt3,k1(2))
      k2(3)=-sign(one/rt3,k1(3))
      k2(4)= sign(one,k1(4))

      call getbase(k1,k2,r1,r2,e1,e2,e3,e4)
      call getq2(nleg,cut2,e1,e2,e3,e4,p0,k1,msq,&
     & q21,q22,q23,q24,q25,q26,q27,q28,q29,q210,qt)
      call getc2(numeval,nleg,rank,c2,cut2,q21,q22,q23,q24,q25,q26,q27,&
     & q28,q29,q210,qt,Vi,msq)
       if (itest.eq.2) call lnntest2(numeval,cut2,c2,qt,p0,e2,e3,e4,ok)
      call store2(icut2,cut2,p0,e1,e2,e3,e4,c2)
      call add2(nleg,c2,cut2,k1,k2,msq,tot2,tot2r,scale2)

      if(verbosity.gt.0)then
         do i=0,9
            write(20,9002) 'c2(',cut2,',',i,')   = (',real(c2(i)),',',aimag(c2(i)),'  )'
         enddo
            write(20,*)
      endif

      do ep=-2,0
      tot(ep)=tot(ep)+tot2(ep)
      enddo
      totr=totr+tot2r

!--- nullify k1,k2,p0
      r1=zip
      r2=zip
       do j=1,4
          e1(j)=0d0
          e2(j)=0d0
          e3(j)=czip
          e4(j)=czip
          k1(j)=0d0
          p0(j)=0d0
       enddo

       icut2=icut2+1
        enddo
       enddo
      nc2=icut2-1

      if (istop.eq.2) goto 99

      if (diff.ge.2) goto 97

       if(verbosity.gt.0)then
         write(20,*) 'Tadpole coefficients: '
      endif


!--- Single cut
      icut1=1
      do j1=0,nleg-1

      cut1=j1

      k1(1)=+one/rt3
      k1(2)=-one/rt3
      k1(3)=+one/rt3
      k1(4)=+two
      k2(1)=+one/rt2
      k2(2)=-one/rt2
      k2(3)=+one/rt2
      k2(4)=+rt3

      do n1=1,4
         p0(n1)=Vi(j1,n1)
      enddo

      call getbase(k1,k2,r1,r2,e1,e2,e3,e4)    
      call getq1(nleg,cut1,e1,e2,e3,e4,p0,msq,q11,q12,q13,q14,q15,qt)
      call getc1(numeval,nleg,rank,c1,cut1,q11,q12,q13,q14,q15,qt,&
     & Vi,msq)
      if (itest.eq.2) call lnntest1(numeval,cut1,c1,qt,p0,e1,e2,e3,e4,ok)
      call store1(icut1,cut1,p0,e1,e2,e3,e4,c1)
      call add1(nleg,c1,cut1,msq,tot1,scale2)

      if(verbosity.gt.0)then
         do i=0,4
            write(20,9002) 'c1(',cut1,',',i,')   = (',real(c1(i)),',',aimag(c1(i)),'  )'
         enddo
            write(20,*)
      endif

      do ep=-2,0
      tot(ep)=tot(ep)+tot1(ep)
      enddo

!--- nullify k1,k2,p0
      r1=zip
      r2=zip
      do j=1,4
          e1(j)=0d0
          e2(j)=0d0
          e3(j)=czip
          e4(j)=czip
          p0(j)=0d0
      enddo

      icut1=icut1+1
      enddo
      nc1=icut1-1

 93   continue
  
      do j=0,9
         c3(j)=0.d0
      enddo
      
 95   continue

      do j=0,9
         c2(j)=0.d0
      enddo
 
 97   continue
 
      do j=0,4
         c1(j)=0.d0
      enddo
      
 99   continue

   
! TEST N=N -------------------------------------------
      if (itest.eq.1) then
         qtest=10.3d0*cone
         mu2test=13.d0
         call nntest(numeval,qtest,mu2test,nleg,Vi,msq,ok)
      endif
!-----------------------------------------------------         

! POWER test ----------------------------------
      if (itest.eq.3) call pwtest(nleg,rank,ok)
! ---------------------------------------------   

      if (itest.eq.0) ok=.true.

      if (ok) then
         
         if(verbosity.gt.0)then
            write(20,*)
            write(20,*)' Result: '
            write(20,*)' Double   Pole = ', tot(-2)
            write(20,*)' Single   Pole = ', tot(-1)
            write(20,*)' Finite   Part = ', tot(0)
            write(20,*)
            write(20,*)'[Rational Part = ', totr,']'
            write(20,*)
            write(20,*)
         endif

      else    

         if(verbosity.gt.0)then
         write(30,*) 'Denominators: '
         do k=0,nleg-1
            write(30,902) ' Pi(',k,') = ', Vi(k,:)
            write(30,903) 'msq(',k,') = ', msq(k)
            write(30,*)
         enddo
         write(30,*)'---------------------------------------- '
         write(30,*)' '
         endif
      endif

 902  format(a4,I1,a4,4(D24.15))
 903  format(a4,I1,a4,1(D24.15))

 9005 format(A3,I5,A6,D24.15,A1,D24.15,A3)
 9004 format(A3,I4,A1,I1,A5,D24.15,A1,D24.15,A3)
 9003 format(A3,I3,A1,I1,A6,D24.15,A1,D24.15,A3)
 9002 format(A3,I2,A1,I1,A7,D24.15,A1,D24.15,A3)



   end subroutine samurai
end module msamurai

